/**
 * @ingroup   sc68app_devel
 * @file      SC68shell_interface.c
 * @author    Ben(jamin) Gerard <ben@sashipa.com>
 * @date      1999/05/13
 * @brief     sc68 - shell "graphic" interface
 * @version   $Id$
 */

/*
 *                        sc68 - shell interface
 *         Copyright (C) 2001 Ben(jamin) Gerard <ben@sashipa.com>
 *
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by the
 *  Free Software Foundation; either version 2 of the License, or (at your
 *  option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#ifdef SC68_SHELL

#include <time.h>

#include "sc68app/SC68os_interface.h"
#include "file68/SC68os_general.h"
#include "io68/ymemul.h"


static char *anim[4] = { "\r- ","\r\\ ","\r| ","\r/ " };
static int ct,oclock;

/*  Open SC68 interface
 *  -id is a pointer to an OS dependant struct
 *   filled by tis call.
 *  return <0 : Error
 *          0 : Not first instance
 *         >0 : Ok
*/
int SC68os_interface_open(char *name, SC68app_t *id)
{
  if(id!=NULL) id = NULL;
  SC68os_pdebug("interface open : name:'%s'\n",name);
  oclock = clock();
  ct = 0;
  return 1;
}

/*  Close SC68 interface
 *  -id : see SC68os_interface_open
*/
int SC68os_interface_close(SC68app_t *id)
{
  id=id;
  SC68os_pdebug("interface close\n");
  return 0;
}

static char *buzzstr[] =
{
	"\\__",
	"\\__",
	"\\__",
	"\\__",
	"/__",
	"/_",
	"/_",
	"/_",
	"\\\\\\",
	"/_",
	"\\/\\",
	"\\__",
	"///",
	"/--",
	"/\\/",
	"/__",
};

static char * volstr[] =
{
	"((O))", /* 0 */
	" (O) ", /* 1 */
	"  O  ", /* 2 */
	"  o  ", /* 3 */
	"  .  ", /* 4 */
};

static void vmeter( int v )
{
  if(v&16)
    SC68os_pdebug(" %s  ",buzzstr[ym.data[0xD]&15]);
  else
  {
    v&=15;
    if(v>=14) v=0;
    else if(v>=12) v=1;
    else if(v>=10) v=2;
    else if(v>=8) v=3;
    else v = 4;
    SC68os_pdebug("%s ",volstr[v]);
  }
}

/*  Call at each sound update ...
 *  Refresh timer counter & whatever you want
*/
int SC68os_interface_update(SC68os_updatecaller_t caller, SC68app_t *id)
{
  int i,c;
  id=id;
  caller(id);
  c=clock();
  if((c-oclock)>=(CLK_TCK/100))
  {
    SC68os_pdebug(anim[(ct++)&3]);
    oclock = c;
    for(i=8; i<11; i++)
    {
      vmeter(ym.data[i]);
    }
    fflush(stderr);
  }
  /* Should not be done like dis !!! */
  return 0;
}


/* Receive a command from this or another SC68 application
 * return 0 : no error, sc68msg->type updated (SC68COM_nop=no message)
*/
int SC68os_interface_receive(SC68app_t *sc68app, SC68msg_t *sc68msg)
{
  return 0;
}

/*  Send a command to this OR another SC68 application
*/
int SC68os_interface_send(SC68app_t *app, SC68msg_t *sc68msg)
{
  return 0;
}

int SC68os_interface_load_cb(SC68app_t *sc68app)
{
  return 0;
}

int SC68os_interface_eject_cb(SC68app_t *sc68app)
{
  return 0;
}

int SC68os_interface_play_cb(SC68app_t *sc68app)
{
  return 0;
}

int SC68os_interface_pause_cb(SC68app_t *sc68app)
{
  return 0;
}

int SC68os_interface_stop_cb(SC68app_t *sc68app)
{
  return 0;
}

int SC68os_interface_ffwd_cb(SC68app_t *sc68app)
{
  return 0;
}

int SC68os_interface_track_cb(SC68app_t *sc68app)
{
  return 0;
}

#endif /* #ifdef SC68_SHELL */

