/**
 * @ingroup   winamp68_devel
 * @file      configure.c
 * @author    Ben(jamin) Gerard <ben@sashipa.com>
 * @date      06/11/2000
 * @brief     sc68 configuration dialog box.
 * @version   $Id$
 */

/*
 *                       sc68 - winamp input plugin
 *         Copyright (C) 2001 Ben(jamin) Gerard <ben@sashipa.com>
 *
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by the
 *  Free Software Foundation; either version 2 of the License, or (at your
 *  option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#include <windows.h>
#include <mmreg.h>
#include <msacm.h>
#include <math.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <commctrl.h>
#include <time.h>
#include "resource.h"
#include "in2.h"

#include "sc68app/SC68app.h"
#include "file68/SC68os_general.h"
#include "file68/SC68utils.h"
#include "file68/SC68file.h"
#include "io68/ymemul.h"
#include "frontend.h" /* Winamp messages def */

extern In_Module mod; // the output module (declared in winamp68.c)

BOOL CALLBACK DialogProc(HWND,UINT,WPARAM,LPARAM);

typedef struct
{
  SC68app_t *app;
  int browse_info;
  int forceTrackOnOff;
  int forceTrackValue;
  int amigaBlend;
  int delayOnOff;
  int delayLR;
  int delayTime;
  int delayForce;
  int delayBufferSz;
  HWND  winamp;

} WA68config_t;

static int idc_voices[3] = {IDC_VOICEA, IDC_VOICEB, IDC_VOICEC};

static int BitsOf(unsigned int v)
{
  unsigned int cnt;
  for (cnt = 0; v > (1u<<cnt); cnt++);
  return cnt;
}

static HWND gHdlg = 0;

// Only exported function. We don't really need an header file for this !
int CreateConfigureDialog(HWND hwnd, SC68app_t *app)
{
static WA68config_t cfg;
  int i;

  if (!gHdlg) {
    i = SC68config_get_id("delay_buffer");

    cfg.winamp          = hwnd; /*mod.hMainWindow ??? marche pas!*/;
    cfg.app             = app;
    cfg.browse_info     = app->browse_info;
    cfg.forceTrackOnOff = !!app->force_track;
    cfg.forceTrackValue = app->force_track ? app->force_track : 1;
    cfg.amigaBlend    = app->mix.amiga_blend;
    cfg.delayOnOff    = app->delay.onoff;
    cfg.delayLR       = app->delay.lr;
    cfg.delayForce    = app->delay.strength;
    cfg.delayTime     = app->delay.time;
    cfg.delayBufferSz = BitsOf((i >= 0) ? app->config[i] : app->delay.buffer_size);

    gHdlg = CreateDialogParam(mod.hDllInstance, (LPCTSTR)IDD_CONFIGURE, hwnd,
                             (DLGPROC)DialogProc,(LPARAM) &cfg);
  }
  return gHdlg ? 0 : -1;
}


static void ApplyNewConfiguration(WA68config_t * cfg)
{
  SC68app_t *app = cfg->app;
  int i;

  /* Browse info */
  app->browse_info = cfg->browse_info;
  if (i = SC68config_get_id("browse_info"), i >= 0)
    app->config[i] = app->browse_info;

  /* First track loaded */
  app->force_track = cfg->forceTrackOnOff ? cfg->forceTrackValue : 0;
  if (i = SC68config_get_id("force_track"), i >= 0)
    app->config[i] = app->force_track;

  /* Amiga blend */
  app->mix.amiga_blend = cfg->amigaBlend;
  if (i = SC68config_get_id("amiga_blend"), i >= 0)
    app->config[i] = app->mix.amiga_blend;

  /* Delay part */
  app->delay.onoff = cfg->delayOnOff;
  if (i = SC68config_get_id("delay_processing"), i >= 0)
    app->config[i] = app->delay.onoff;

  app->delay.lr    = cfg->delayLR;
  if (i = SC68config_get_id("delay_lr"), i >= 0)
    app->config[i] = app->delay.lr;

  app->delay.strength = cfg->delayForce;
  if (i = SC68config_get_id("delay_strength"), i >= 0)
    app->config[i] = app->delay.strength;

  app->delay.time     = cfg->delayTime;
  if (i = SC68config_get_id("delay_time"), i >= 0)
    app->config[i] = app->delay.time;

  if (i = SC68config_get_id("delay_buffer"), i >= 0)
    app->config[i] = 1 << cfg->delayBufferSz;

  SC68config_save(&app->config,
                  SC68os_resource_name(SC68rsc_config, "config"));
}

static int SetText(HWND hdlg, int id, const char *new_text)
{
  LRESULT res;
  WPARAM wParam = 0;
  LPARAM lParam = (LPARAM)new_text;
  res = SendDlgItemMessage(hdlg, id, WM_SETTEXT, wParam, lParam);
  return (!!res);
}

static int SetIconText(HWND hdlg, int id, const char *new_text)
{
  return SetText(hdlg, id, new_text);
}

static int SetIconFormat(HWND hdlg, int id, const char *format, int n)
{
  char buffer[64];
  _snprintf(buffer,sizeof(buffer), format, n);
  buffer[sizeof(buffer)-1] = 0;
  return SetText(hdlg, id, buffer);
}


static int SetCheck(HWND hdlg, int id, int enable)
{
  LRESULT res;
  res = SendDlgItemMessage(hdlg, id, BM_SETCHECK, !!enable, 0);
  return (!!res);
}

static int GetCheck(HWND hdlg, int id)
{
  LRESULT res;
  res = SendDlgItemMessage(hdlg, id, BM_GETCHECK, 0, 0);
  return res == BST_CHECKED;
}



static int SetEnable(HWND hdlg, int id, int enable)
{
  LRESULT res;
  res = EnableWindow(GetDlgItem(hdlg,id),!!enable);
  return (!!res);
}

static int ChangeBIDB(HWND hdlg, WA68config_t *cfg)
{
  return SetCheck(hdlg, IDC_BIDB, !!cfg->browse_info);
}

static int ChangeYMvoices(HWND hdlg)
{
  int i;
  int activ = YM_get_activevoices();
  for (i=0; i<3; ++i) {
    SetCheck(hdlg, idc_voices[i], (activ>>i)&1);
  }
  return activ;
}

static int ChangeForceTrack(HWND hdlg, WA68config_t *cfg)
{
  if (cfg->forceTrackValue < 0) {
    cfg->forceTrackValue = 0;
  } else if (cfg->forceTrackValue > 99) {
    cfg->forceTrackValue = 99;
  }

  SetCheck(hdlg, IDC_FORCETRKONOFF,!!cfg->forceTrackOnOff);
  SetEnable(hdlg, IDC_FORCETRK,    !!cfg->forceTrackOnOff);
  SetEnable(hdlg, IDC_FORCETRKUP,  !!cfg->forceTrackOnOff);
  SendDlgItemMessage(hdlg, IDC_FORCETRKUP,UDM_SETPOS, 0, (LPARAM) MAKELONG((short)cfg->forceTrackValue , 0));
  SetIconFormat(hdlg, IDC_FORCETRK, "%02d", cfg->forceTrackValue);
  return 0;
}

static int ChangeAmigaBlend(HWND hdlg, WA68config_t *cfg)
{
  if (cfg->amigaBlend < 0) {
    cfg->amigaBlend = 0;
  } else if (cfg->amigaBlend > 65536) {
    cfg->amigaBlend = 65536;
  }

  if (cfg->app) {
    cfg->app->mix.amiga_blend = cfg->amigaBlend;
  }

  SendDlgItemMessage(hdlg, IDC_AGABLEND, TBM_SETPOS, 1, cfg->amigaBlend >> 6);

  return 0;
}

static int ChangeDelay(HWND hdlg, WA68config_t * cfg)
{
  cfg->delayOnOff = !!cfg->delayOnOff;
  cfg->delayLR    = !!cfg->delayLR;

  if (cfg->app) {
    /* Apply pseudo realtime FX */
    cfg->app->delay.onoff = cfg->delayOnOff;
    cfg->app->delay.lr    = cfg->delayLR;
    cfg->app->delay.time  = cfg->delayTime;
    cfg->app->delay.strength = cfg->delayForce;
  }
  SetCheck(hdlg, IDC_DELAYONOFF,      cfg->delayOnOff);
  SetCheck(hdlg, IDC_DELAYLR,         cfg->delayLR);
  SetEnable(hdlg, IDC_DELAYTIME,      cfg->delayOnOff);
  SetEnable(hdlg, IDC_TIMELBL  ,      cfg->delayOnOff);
  SetEnable(hdlg, IDC_DELAYSTRENGTH,  cfg->delayOnOff && !cfg->delayLR);
  SetEnable(hdlg, IDC_STRENGTHLBL  ,  cfg->delayOnOff && !cfg->delayLR);

  SendDlgItemMessage(hdlg, IDC_DELAYSZUP, UDM_SETPOS, 0, (LPARAM) MAKELONG((short)cfg->delayBufferSz, 0));
  SetIconFormat(hdlg, IDC_DELAYSZ,"%d", 1 << cfg->delayBufferSz);

  return 0;
}

static int SetSlider(HWND hdlg, int id, int pos, int min, int max)
{
  SendDlgItemMessage(hdlg, id, TBM_SETTICFREQ,  (max-min) / 4, 0);
  SendDlgItemMessage(hdlg, id, TBM_SETRANGEMIN, 0, min);
  SendDlgItemMessage(hdlg, id, TBM_SETRANGEMAX, 0, max);
  SendDlgItemMessage(hdlg, id, TBM_SETPOS,      1, pos);
  return 0;
}

static WA68config_t *ChangeConfig(HWND hdlg, WA68config_t *cfg)
{
  WA68config_t *old = (WA68config_t* )SetWindowLong(hdlg, DWL_USER, (LONG)cfg);
  SendDlgItemMessage(hdlg, IDC_FORCETRKUP, UDM_SETRANGE32, 1, 99);
  ChangeForceTrack(hdlg, cfg);

  ChangeBIDB(hdlg, cfg);

  SetSlider(hdlg, IDC_AGABLEND,       cfg->amigaBlend, 0, 1024);
  ChangeAmigaBlend(hdlg, cfg);

  SendDlgItemMessage(hdlg, IDC_DELAYSZUP, UDM_SETRANGE32, 11, 14);
  SetSlider(hdlg, IDC_DELAYTIME,      (cfg->delayTime<<2) + 128, 0, 256);
  SetSlider(hdlg, IDC_DELAYSTRENGTH,  cfg->delayForce, 16, 240);
  ChangeDelay(hdlg, cfg);
  ChangeYMvoices(hdlg);

  return old;
}

static WA68config_t *GetConfig(HWND hdlg)
{
  return (WA68config_t *)GetWindowLong(hdlg, DWL_USER);
}


static BOOL CALLBACK DialogProc(
  HWND hdlg,      // handle to dialog box
  UINT uMsg,      // message
  WPARAM wParam,  // first message parameter
  LPARAM lParam   // second message parameter
)
{
  WA68config_t *cfg = GetConfig(hdlg);

  switch(uMsg) {
		case WM_INITDIALOG:
      {
        cfg = (WA68config_t *)lParam;
        ChangeConfig(hdlg, cfg);

        ShowWindow(hdlg, SW_SHOW);
        UpdateWindow(hdlg);
      } return 1;

    case WM_COMMAND:
      {
        int wNotifyCode = HIWORD(wParam); // notification code
        int wID = LOWORD(wParam);         // item, control, or accelerator identifier
        HWND hwndCtl = (HWND) lParam;     // handle of control

        switch(wID) {
          case IDOK:
            ApplyNewConfiguration(cfg);
          case IDCANCEL:
            DestroyWindow (hdlg);
            //EndDialog(hdlg, wID==IDOK ? 0 : 1);
            break;
          case IDC_FORCETRKONOFF:
            switch (wNotifyCode ) {
            case BN_CLICKED:
              if (cfg) {
                cfg->forceTrackOnOff ^= 1;
                ChangeForceTrack(hdlg, cfg);
              }
              break;
            }
            break;
          case IDC_BIDB:
            switch (wNotifyCode ) {
            case BN_CLICKED:
              if (cfg) {
                cfg->browse_info ^= 1;
                ChangeBIDB(hdlg, cfg);
              }
              break;
            }
            break;

          case IDC_VOICEA:
          case IDC_VOICEB:
          case IDC_VOICEC:
            {
              int activ,i;
              switch (wNotifyCode ) {
                case BN_CLICKED:
                  for (activ=i=0; i<3; ++i) {
                    activ |= GetCheck(hdlg, idc_voices[i]) << i;
                  }
                  YM_set_activeVoices(activ);
              }
            }
            break;

        case IDC_BUTTON1:
          switch (wNotifyCode ) {
          case BN_CLICKED:
            {
              HWND winamp = GetParent(GetParent(hdlg));
              const char *url = "http://sashipa.ben.free.fr/sc68";
              if (SendMessage(winamp,WM_WA_IPC,0,IPC_INETAVAILABLE) == 1) {
                SendMessage(winamp,WM_WA_IPC,0,IPC_MBOPENREAL);
                SendMessage(winamp,WM_WA_IPC,(WPARAM)url, IPC_MBOPENREAL);
              }
            } break;


/*#define IPC_MBOPEN 241*/
/* (requires Winamp 2.05+)
** SendMessage(hwnd_winamp,WM_WA_IPC,0,IPC_MBOPEN);
** SendMessage(hwnd_winamp,WM_WA_IPC,(WPARAM)url,IPC_MBOPEN);
**
** IPC_MBOPEN will open a new URL in the minibrowser. if url is NULL, it will open the Minibrowser window.
*/

            break;
          }
          break;



          case IDC_DELAYONOFF:
          case IDC_DELAYLR:
            switch (wNotifyCode ) {
            case BN_CLICKED:
              if (cfg) {
                if (wID == IDC_DELAYONOFF)
                  cfg->delayOnOff ^= 1;
                if (wID == IDC_DELAYLR)
                  cfg->delayLR ^= 1;

                ChangeDelay(hdlg, cfg);
              }
              break;
            }
            break;


        }
      } return 0;

    case WM_VSCROLL:
    case WM_HSCROLL:
      {
        int nScrollCode = (int) LOWORD(wParam);   // scroll bar value
        int nPos = (short int) HIWORD(wParam);    // scroll box position
        HWND hwndScrollBar = (HWND) lParam;       // handle to scroll bar


        if (hwndScrollBar == GetDlgItem(hdlg, IDC_FORCETRKUP)) {
          if (cfg) {
            cfg->forceTrackValue = nPos;
            ChangeForceTrack(hdlg,cfg);
          }
        } else if (hwndScrollBar == GetDlgItem(hdlg, IDC_DELAYSZUP)) {
          if (cfg) {
            cfg->delayBufferSz = nPos;
            ChangeDelay(hdlg,cfg);
          }
        } else if (hwndScrollBar == GetDlgItem(hdlg, IDC_AGABLEND)) {
          nPos = SendMessage(hwndScrollBar, TBM_GETPOS , 0 , 0);
          if (cfg) {
            cfg->amigaBlend = (nPos << 6) | (nPos >> 4);
            ChangeAmigaBlend(hdlg, cfg);
          }
        } else if (hwndScrollBar == GetDlgItem(hdlg, IDC_DELAYTIME)) {
          nPos = SendMessage(hwndScrollBar, TBM_GETPOS , 0 , 0);
          if (cfg) {
            cfg->delayTime = (nPos - 128) >> 2;
            ChangeDelay(hdlg,cfg);
          }
        } else if (hwndScrollBar == GetDlgItem(hdlg, IDC_DELAYSTRENGTH)) {
          nPos = SendMessage(hwndScrollBar, TBM_GETPOS , 0 , 0);
          if (cfg) {
            cfg->delayForce = nPos;
            ChangeDelay(hdlg,cfg);
          }
        }
      } return 0;

		case WM_DESTROY:
      {
        gHdlg = 0;
      } return 0;

		case WM_CLOSE:
      EndDialog(hdlg, 1);
			return 0;

  }

  return 0;
}



